<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use App\Model\Shopify\Order;
use App\Model\RechargeApi;
use Config;
use Log;

class OrderCancelAndRefundJob implements ShouldQueue {

    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    public $webhookData;
    private $orderObj;
    public $rechargeObj;

    /**
     * Create a new job instance.
     */
    public function __construct($webhookDetails) {

        $this->webhookData = $webhookDetails;
    }

    /**
     * Execute the job.
     */
    public function handle() {

        $this->orderObj = new Order();  
        $this->rechargeObj = new RechargeApi();
        $orderId = $this->webhookData->id;
        $orderCancelPromos = ["DEL-TEST-9F7JE1"];
        $orderCancelPromoStatus = 0;
        $orderOtpStatus = 0;
        $orderTags = '';
        $checkOrderTagStatus = '';

        Log::channel('automatedOrderCancelAndRefundLog')->info("Webhook processing started");

        $orderDiscountData = !empty($this->webhookData->discount_codes) ? $this->webhookData->discount_codes : '';

        if (empty($orderDiscountData)) {

            Log::channel('automatedOrderCancelAndRefundLog')->info("Not found any promo codes in the Shopify order,
            (OrderId:".$orderId);
            return null;
        }

        $orderCancelPromos = array_map('trim', $orderCancelPromos);
        $orderCancelPromos = array_map('strtolower', $orderCancelPromos);

        foreach ($orderDiscountData as $discountKey => $discountValue) {

            $discountCode = '';
            $discountCode = strtolower(trim($discountValue->code));
            if (in_array($discountCode, $orderCancelPromos)) {
                $orderCancelPromoStatus = 1;
            }
        }

        if ($orderCancelPromoStatus == 0) {

            Log::channel('automatedOrderCancelAndRefundLog')->info("Not found the order cancel required promo codes in 
            the Shopify order, OrderId:".$orderId);
            return null;
        }

        if ($orderCancelPromoStatus == 1) {

            $lineItemId = !empty($this->webhookData->line_items[0]->id) ?
            $this->webhookData->line_items[0]->id : '';
            $productQuantity = !empty($this->webhookData->line_items[0]->quantity) ?
            $this->webhookData->line_items[0]->quantity : '';
            $orderTags = !empty($this->webhookData->tags) ?
            $this->webhookData->tags : '';
            $productVariantId = !empty($this->webhookData->line_items[0]->variant_id) ?
            $this->webhookData->line_items[0]->variant_id : '';
            if (!empty($orderTags)) {
                $checkOrderTagStatus = $this->checkOrderTagWithOtp($orderTags);
            }

            $orderAutomatedTagStatus = $this->checkOrderTagWithAutomated($orderTags);
            if ($orderAutomatedTagStatus) {

                Log::channel('automatedOrderCancelAndRefundLog')->info("Already cancel and refund the Shopify order, 
                OrderId:".$orderId);
                return null;
            }

            // Get charge details from recharge by shopify order ID
            $chargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($orderId);
            if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
                empty($chargeDetails) || empty($chargeDetails->charges)) {

                $rechargeError  = "Unknown error";
                if (!empty($chargeDetails->error)) {

                    $rechargeError = json_encode($chargeDetails->error);
                } else if (!empty($chargeDetails->errors)) {

                    $rechargeError = json_encode($chargeDetails->errors);
                } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                    $rechargeError = 'Failed to get the charge details by using the given orderId: '.$orderId;
                }
                $returnData = [
                    'error' => $rechargeError
                ];

                Log::channel('automatedOrderCancelAndRefundLog')->info("Failed to get the charge details from 
                the recharge(OrderId:".$orderId."), Error: (".$rechargeError.")");
                return null;
                
            }

            $chargeId = $chargeDetails->charges[0]->id;
            $subscriptionId = $chargeDetails->charges[0]->line_items[0]->subscription_id;
            $rechargeChargeAmount = $chargeDetails->charges[0]->total_price;
            $rechargeHashToken = trim($chargeDetails->charges[0]->customer_hash);

            // Get shopify order transactional details
            $transactionalDetails = $this->orderObj->getOrderTransactionalDetails($orderId);
            if (!empty($transactionalDetails['error']) || !empty($transactionalDetails['errors']) ||
                empty($transactionalDetails) || empty($transactionalDetails['transactions'])) {

                $shTransactionalError  = "Unknown error";
                if (!empty($transactionalDetails['error'])) {

                    $shTransactionalError = json_encode($transactionalDetails['error']);
                } else if (!empty($transactionalDetails['errors'])) {

                    $shTransactionalError = json_encode($transactionalDetails['errors']);
                } else if (empty($transactionalDetails) || empty($transactionalDetails['transactions'])) {

                    $shTransactionalError = 'Failed to get the shopify transaction details by using the given 
                    orderId: '.$orderId;
                }
                $returnData = [
                    'error' => $shTransactionalError
                ];

                Log::channel('automatedOrderCancelAndRefundLog')->info("Failed to get the Shopify order 
                transactional details(OrderId:".$orderId."), Error: (".$shTransactionalError.")");
                return null;
                
            }
            $transactionalId = '';
            $transactionalGateWay = '';
            foreach ($transactionalDetails['transactions'] as $transKey => $transValue) {

                if ($transValue['kind'] == 'sale') {
                    $transactionalId = $transValue['id'];
                    $transactionalGateWay = $transValue['gateway'];
                }
            }

            // Refund shopify order via shopify API
            $shopifyRefundRequest = [];
            $shopifyRefundRequest = array(
                "refund" => array(
                    "currency" => "USD",
                    "notify" => true,
                    "shipping" => array(
                        "full_refund" => true
                    ),
                    "refund_line_items" => array(
                        array(
                            "line_item_id" => $lineItemId,
                            "quantity" => $productQuantity,
                            "line_item" => array(
                                "variant_id" => $productVariantId
                            )
                        )
                    ),
                    "transactions" => array(
                        array(
                            "parent_id" => $transactionalId,
                            "amount" => $rechargeChargeAmount,
                            "kind" => "refund",
                            "gateway" => $transactionalGateWay
                        )
                    )
                )
            );

            $refundResponse = $this->orderObj->refundOrder($orderId, $shopifyRefundRequest);
            if (!empty($refundResponse['error']) || !empty($refundResponse['errors']) ||
                empty($refundResponse)) {

                $shopifyRefundError  = "Unknown error";
                if (!empty($refundResponse['error'])) {

                    $shopifyRefundError = json_encode($refundResponse['error']);
                } else if (!empty($refundResponse['errors'])) {

                    $shopifyRefundError = json_encode($refundResponse['errors']);
                } else if (empty($refundResponse)) {

                    $shopifyRefundError = 'The refunded response from Shopify is empty';
                }

                $returnData = [
                    'error' => $shopifyRefundError
                ];

                Log::channel('automatedOrderCancelAndRefundLog')->info("Failed to refund the order from Shopify
                (OrderId:".$orderId."), Error: (".$shopifyRefundError.")");
                return null;
            }

            // Refund recharge charge
            $refundAmountRequest = array(
                "amount" => $rechargeChargeAmount
            );
            $chargeRefundedDetails = $this->rechargeObj->refundRechargeChargeById($chargeId, json_encode($refundAmountRequest));
            if (!empty($chargeRefundedDetails->error) || !empty($chargeRefundedDetails->errors) ||
                empty($chargeRefundedDetails)) {

                $rechargeRefundError  = "Unknown error";
                if (!empty($chargeRefundedDetails->error)) {

                    $rechargeRefundError = json_encode($chargeRefundedDetails->error);
                } else if (!empty($chargeRefundedDetails->errors)) {

                    $rechargeRefundError = json_encode($chargeRefundedDetails->errors);
                } else if (empty($chargeRefundedDetails)) {

                    $rechargeRefundError = 'The refunded response from recharge is empty';
                }

                $returnData = [
                    'error' => $rechargeRefundError
                ];

                Log::channel('automatedOrderCancelAndRefundLog')->info("Failed to refund the charge from recharge
                (OrderId:".$orderId."), Error: (".$rechargeRefundError.")");
                return null;
            }

            // Cancel shopify order
            $orderCancelledDetails = $this->orderObj->cancelOrder($orderId);
            if (!empty($orderCancelledDetails['error']) || !empty($orderCancelledDetails['errors']) ||
                empty($orderCancelledDetails)) {

                $orderCancelError  = "Unknown error";
                if (!empty($orderCancelledDetails['error'])) {

                    $orderCancelError = json_encode($orderCancelledDetails['error']);
                } else if (!empty($orderCancelledDetails['errors'])) {

                    $orderCancelError = json_encode($orderCancelledDetails['errors']);
                } else if (empty($orderCancelledDetails)) {

                    $orderCancelError = 'The order canceled response from Shopify is empty';
                }

                $returnData = [
                    'error' => $orderCancelError
                ];

                Log::channel('automatedOrderCancelAndRefundLog')->info("Failed to cancel the order from Shopify
                (OrderId:".$orderId."), Error: (".$orderCancelError.")");
                return null;
            }

            if (!$checkOrderTagStatus) {

                // Cancel recharge subscription
                $cancelRequest = array(
                    "cancellation_reason" => "other reason"
                );
                $subscriptionCancelledDetails = $this->rechargeObj->cancelRechargeSubscription($subscriptionId, 
                json_encode($cancelRequest));
                if (!empty($subscriptionCancelledDetails->error) || !empty($subscriptionCancelledDetails->errors) ||
                    empty($subscriptionCancelledDetails)) {

                    $subscriptionCancelError  = "Unknown error";
                    if (!empty($subscriptionCancelledDetails->error)) {

                        $subscriptionCancelError = json_encode($subscriptionCancelledDetails->error);
                    } else if (!empty($subscriptionCancelledDetails->errors)) {

                        $subscriptionCancelError = json_encode($subscriptionCancelledDetails->errors);
                    } else if (empty($subscriptionCancelledDetails)) {

                        $subscriptionCancelError = 'The subscription cancelled response from Shopify is empty';
                    }

                    $returnData = [
                        'error' => $subscriptionCancelError
                    ];

                    Log::channel('automatedOrderCancelAndRefundLog')->info("Failed to cancel the subscription from Recharge
                    (SubscriptionId:".$subscriptionId."), Error: (".$subscriptionCancelError.")");
                    return null;
                }
            }

            // Update Shopify order tags
            $existingOrderTags = [];
            if (!empty($orderTags)) {
                $existingOrderTags = explode(",", $orderTags);
                $existingOrderTags = array_map('trim', $existingOrderTags);
            }

            $updatedShopifyTags = '';
            $shopifyTagsToUpdate = [];
            $shopifyTagsToUpdate = $existingOrderTags;
            $shopifyTagsToUpdate[] = 'automated-test-order';
            $updatedShopifyTags = implode(",", $shopifyTagsToUpdate);

            $orderUpdateRequest = [];
            $orderUpdateRequest = array (
                "order" => array (
                    "id" => $orderId,
                    "tags" => $updatedShopifyTags
                )
            );

            // Update order tags to Shopify
            $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $orderId);
            if (!empty($shopifyOrderUpdateResponse['error']) || !empty($shopifyOrderUpdateResponse['errors']) ||
                empty($shopifyOrderUpdateResponse)) {

                $shopifyError  = "Unknown error";
                if (!empty($shopifyOrderUpdateResponse['error'])) {

                    $shopifyError = json_encode($shopifyOrderUpdateResponse['error']);
                } else if (!empty($shopifyOrderUpdateResponse->errors)) {

                    $shopifyError = json_encode($shopifyOrderUpdateResponse['errors']);
                } else if (empty($shopifyOrderUpdateResponse)) {

                    $shopifyError = 'Failed to update the Shopify order';
                }
                $returnData = [
                    'error' => $shopifyError
                ];

                Log::channel('automatedOrderCancelAndRefundLog')->info("Failed to update the Shopify order(orderUpdateRequest:".
                json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
                return null;
            }

            Log::channel('automatedOrderCancelAndRefundLog')->info("Successfully cancelled and refunded the order, OrderId:".$orderId);
        }
        Log::channel('automatedOrderCancelAndRefundLog')->info("Webhook processing completed");
    }

    /**
     * Check Shopify order tags
     */
    public function checkOrderTagWithOtp($orderTags) {

        $explodeTags = explode(",", $orderTags);
        $explodeTags = array_map('trim', $explodeTags);
        $explodeTags = array_map('strtolower', $explodeTags);
        if (in_array("otp", $explodeTags)) {
            return true;
        }
        return false;
    }

    /**
     * Check to cancel and refund order tag status
     */
    public function checkOrderTagWithAutomated($orderTags) {

        $explodeTags = explode(",", $orderTags);
        $explodeTags = array_map('trim', $explodeTags);
        $explodeTags = array_map('strtolower', $explodeTags);
        if (in_array("automated-test-order", $explodeTags)) {
            return true;
        }
        return false;
    }
}
